@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* Copyright (C) 2002 the late Joey Shutup.
  *
  * http://www.streetmap.co.uk/streetmap.dll?postcode2map?BS24+9TZ
@@ -22,7 +23,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
 #include "config.h"
 
@@ -48,17 +49,17 @@
 #include "execcontext.h"
 #include "builtins.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
 extern int nColorsets;  /* in libs/Colorset.c */
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
 /* When fvwm destroys pixmaps it puts them on a list and only destroys them
  * after some period of inactivity. This is necessary because changing colorset
@@ -80,9 +81,9 @@ struct root_pic
 	int height;
 };
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
 static char *black = "black";
 static char *white = "white";
@@ -162,32 +163,40 @@ static char *csetopts[] =
 	NULL
 };
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 static
 Pixmap get_root_pixmap(Atom prop)
 {
 	Atom type;
 	int format;
 	unsigned long length, after;
-	unsigned char *data;
+	unsigned char *reteval = NULL;
 	int ret;
+	Pixmap pix = None;
 
-	ret = XGetWindowProperty(dpy, Scr.Root, prop, 0L, 1L, False, XA_PIXMAP,
-			   &type, &format, &length, &after, &data);
-	if (ret == Success && type == XA_PIXMAP && format == 32 && length == 1 &&
-	    after == 0)
+	ret = XGetWindowProperty(
+		dpy, Scr.Root, prop, 0L, 1L, False, XA_PIXMAP,
+		&type, &format, &length, &after, &reteval);
+	if (
+		ret == Success && type == XA_PIXMAP && format == 32 &&
+		length == 1 && after == 0)
 	{
-		return *((Pixmap *)data);
+		pix = (Pixmap)(*(long *)reteval);
 	}
-	return None;
+	if (reteval)
+	{
+		XFree(reteval);
+	}
+	return pix;
 }
 
 void update_root_pixmap(Atom prop)
 {
 	static Atom a_rootpix = None;
-	unsigned int w = 0, h = 0;
+	int w = 0;
+	int h = 0;
 	XID dummy;
 	Pixmap pix;
 
@@ -201,7 +210,8 @@ void update_root_pixmap(Atom prop)
 		pix = get_root_pixmap(prop);
 		if (pix && !XGetGeometry(
 			dpy, pix, &dummy, (int *)&dummy, (int *)&dummy,
-			&w, &h, (unsigned int *)&dummy, (unsigned int *)&dummy))
+			(unsigned int *)&w, (unsigned int *)&h,
+			(unsigned int *)&dummy, (unsigned int *)&dummy))
 		{
 			pix = None;
 		}
@@ -211,7 +221,8 @@ void update_root_pixmap(Atom prop)
 		pix = get_root_pixmap(a_rootpix);
 		if (pix && !XGetGeometry(
 			dpy, pix, &dummy, (int *)&dummy, (int *)&dummy,
-			&w, &h, (unsigned int *)&dummy, (unsigned int *)&dummy))
+			(unsigned int *)&w, (unsigned int *)&h,
+			(unsigned int *)&dummy, (unsigned int *)&dummy))
 		{
 			pix = None;
 		}
@@ -572,9 +583,9 @@ static void parse_simple_tint(
 	}
 }
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
-void cleanup_colorsets()
+void cleanup_colorsets(void)
 {
 	struct junklist *oldjunk = junk;
 
@@ -643,11 +654,13 @@ void parse_colorset(int n, char *line)
 	alloc_colorset(n);
 	cs = &Colorset[n];
 
-	/* ---------- Parse the options ---------- */
+	/*** Parse the options ***/
 	while (line && *line)
 	{
-		/* Read next option specification delimited by a comma or \0. */
-		line = GetQuotedString(line, &optstring, ",", NULL, NULL, NULL);
+		/* Read next option specification delimited by a comma or \0.
+		 */
+		line = GetQuotedString(
+			line, &optstring, ",", NULL, NULL, NULL);
 		if (!optstring)
 			break;
 		args = GetNextToken(optstring, &option);
@@ -688,7 +701,8 @@ void parse_colorset(int n, char *line)
 			has_sh_changed = True;
 			break;
 		case 12: /* fgsh */
-			get_simple_color(args, &fgsh, cs, FGSH_SUPPLIED, 0,NULL);
+			get_simple_color(
+				args, &fgsh, cs, FGSH_SUPPLIED, 0,NULL);
 			has_fgsh_changed = True;
 			break;
 		case 13: /* fg_alpha */
@@ -993,7 +1007,9 @@ void parse_colorset(int n, char *line)
 	/*
 	 * reset the pixmap if the tint or the alpha has changed
 	 */
-	if (!do_reload_pixmap && (has_tint_changed || has_image_alpha_changed))
+	if (!do_reload_pixmap &&
+	    (has_tint_changed || has_image_alpha_changed  ||
+	     (has_bg_changed && cs->alpha_pixmap != None)))
 	{
 		if (cs->pixmap_type == PIXMAP_ROOT_PIXMAP_PURE ||
 		    cs->pixmap_type == PIXMAP_ROOT_PIXMAP_TRAN)
@@ -1067,13 +1083,19 @@ void parse_colorset(int n, char *line)
 
 		if (cs->color_flags & BG_AVERAGE)
 		{
-			average_pix =
-			   (cs->picture != NULL &&
-			    cs->picture->picture != None) ?
-				cs->picture->picture : cs->pixmap;
+			if (cs->picture != NULL && cs->picture->picture != None)
+			{
+				average_pix = cs->picture->picture;
+			}
+			else if (cs->pixmap != ParentRelative)
+			{
+				average_pix = cs->pixmap;
+			}
+
 			if (average_pix == root_pic.pixmap)
 			{
-				int w,h;
+				int w;
+				int h;
 				XID dummy;
 
 				XGrabServer(dpy);
@@ -1086,9 +1108,12 @@ void parse_colorset(int n, char *line)
 				{
 					average_pix = None;
 				}
-				if (w != cs->width || h != cs->height)
+				else
 				{
-					average_pix = None;
+					if (w != cs->width || h != cs->height)
+					{
+						average_pix = None;
+					}
 				}
 				if (average_pix == None)
 				{
@@ -1111,6 +1136,9 @@ void parse_colorset(int n, char *line)
 
 			has_bg_changed = True;
 			/* create an array to store all the pixmap colors in */
+			/* Note: this may allocate a lot of memory:
+			 * cs->width * cs->height * 12 and then the rest of the
+			 * procedure can take a lot of times */
 			colors = (XColor *)safemalloc(
 				cs->width * cs->height * sizeof(XColor));
 			/* get the pixmap and mask into an image */
@@ -1198,7 +1226,8 @@ void parse_colorset(int n, char *line)
 					Pixel old_bg = cs->bg;
 
 					PictureFreeColors(
-						dpy, Pcmap, &cs->bg, 1, 0, True);
+						dpy, Pcmap, &cs->bg, 1, 0,
+						True);
 					PictureAllocColor(
 						dpy, Pcmap, &color, True);
 					cs->bg = color.pixel;
@@ -1500,7 +1529,8 @@ void parse_colorset(int n, char *line)
 	 * ------- change the masked out parts of the background pixmap -------
 	 */
 	if (cs->pixmap != None && cs->pixmap != ParentRelative &&
-	    (!CSETS_IS_TRANSPARENT_ROOT(cs)||cs->allows_buffered_transparency) &&
+	    (!CSETS_IS_TRANSPARENT_ROOT(cs)||
+	     cs->allows_buffered_transparency) &&
 	    (cs->mask != None || cs->alpha_pixmap != None ||
 	     cs->image_alpha_percent < 100 || cs->tint_percent > 0) &&
 	    (has_pixmap_changed || has_bg_changed || has_image_alpha_changed
@@ -1632,11 +1662,11 @@ void parse_colorset(int n, char *line)
 	return;
 }
 
-/*****************************************************************************
+/*
  * alloc_colorset() grows the size of the Colorset array to include set n
  * colorset_t *Colorset will be altered
  * returns the address of the member
- *****************************************************************************/
+ */
 void alloc_colorset(int n)
 {
 	/* do nothing if it already exists */
@@ -1753,3 +1783,31 @@ void CMD_ReadWriteColors(F_CMD_ARGS)
 
 	return;
 }
+
+void CMD_Colorset(F_CMD_ARGS)
+{
+	int n;
+	char *token;
+
+	if (GetIntegerArguments(action, &token, &n, 1) != 1)
+	{
+		return;
+	}
+	if (n < 0)
+	{
+		return;
+	}
+	if (token == NULL)
+	{
+		return;
+	}
+	parse_colorset(n, token);
+	update_fvwm_colorset(n);
+
+	return;
+}
+
+void CMD_CleanupColorsets(F_CMD_ARGS)
+{
+	cleanup_colorsets();
+}
