@@ -254,7 +254,19 @@ static Window __button_proxy = 0;
 
 
 /* how many desks does gnome know about */
-static unsigned int gnome_max_desk = 1;
+static int gnome_max_desk = 1;
+
+static int atom_size(int format)
+{
+	if (format == 32)
+	{
+		return sizeof(long);
+	}
+	else
+	{
+		return (format >> 3);
+	}
+}
 
 static void *
 AtomGet(Window win, Atom to_get, Atom type, int *size)
@@ -269,15 +281,18 @@ AtomGet(Window win, Atom to_get, Atom type, int *size)
 	retval = NULL;
 	length = 0x7fffffff;
 	XGetWindowProperty(
-		dpy, win, to_get, 0, length, False, type, &type_ret,
+		dpy, win, to_get, 0L, length, False, type, &type_ret,
 		&format_ret, &num_ret, &bytes_after, &retval);
 
 	if ((retval) && (num_ret > 0) && (format_ret > 0))
 	{
-		data = safemalloc(num_ret * (format_ret >> 3));
+		int asize;
+
+		asize = atom_size(format_ret);
+		data = safemalloc(num_ret * asize);
 		if (data)
 		{
-			memcpy(data, retval, num_ret * (format_ret >> 3));
+			memcpy(data, retval, num_ret * asize);
 		}
 		XFree(retval);
 		*size = num_ret * (format_ret >> 3);
@@ -295,9 +310,9 @@ static void
 GNOME_GetHintIcons(FvwmWindow *fwin)
 {
 	Atom atom_get;
-	CARD32 *retval;
+	long *retval;
 	int size;
-	unsigned int i;
+	int i;
 	Pixmap pmap;
 	Pixmap mask;
 
@@ -305,7 +320,10 @@ GNOME_GetHintIcons(FvwmWindow *fwin)
 	retval = AtomGet(FW_W(fwin), atom_get, XA_PIXMAP, &size);
 	if (retval)
 	{
-		for (i = 0; i < (size / (sizeof(CARD32))); i += 2)
+		int n;
+
+		n = size / atom_size(32);
+		for (i = 0; i < n; i += 2)
 		{
 			pmap = retval[i];
 			mask = retval[i + 1];
@@ -320,7 +338,7 @@ static void
 GNOME_GetHintLayer(FvwmWindow *fwin)
 {
 	Atom atom_get;
-	CARD32 *retval;
+	long *retval;
 	int size;
 
 	atom_get = XInternAtom(dpy, XA_WIN_LAYER, False);
@@ -338,7 +356,7 @@ static void
 GNOME_GetHintState(FvwmWindow *fwin)
 {
 	Atom atom_get;
-	CARD32 *retval;
+	long *retval;
 	int size;
 
 	atom_get = XInternAtom(dpy, XA_WIN_STATE, False);
@@ -449,7 +467,7 @@ static void
 GNOME_GetExpandedSize(FvwmWindow *fwin)
 {
 	Atom atom_get;
-	CARD32 *retval;
+	long *retval;
 	int size;
 
 	/* unimplimented */
@@ -492,7 +510,7 @@ void
 GNOME_SetHints(FvwmWindow *fwin)
 {
 	Atom atom_set;
-	int val;
+	long val;
 
 	atom_set = XInternAtom(dpy, XA_WIN_STATE, False);
 	val = 0;
@@ -620,7 +638,7 @@ void
 GNOME_SetDesk(FvwmWindow *fwin)
 {
 	Atom atom_set;
-	int val;
+	long val;
 
 	atom_set = XInternAtom(dpy, XA_WIN_WORKSPACE, False);
 	val = fwin->Desk;
@@ -640,7 +658,7 @@ void
 GNOME_SetLayer(FvwmWindow *fwin)
 {
 	Atom atom_set;
-	int val;
+	long val;
 
 	atom_set = XInternAtom(dpy, XA_WIN_LAYER, False);
 	val = get_layer(fwin);
@@ -659,7 +677,7 @@ void
 GNOME_SetAreaCount(void)
 {
 	Atom atom_set;
-	CARD32 val[2];
+	long val[2];
 
 	atom_set = XInternAtom(dpy, XA_WIN_AREA_COUNT, False);
 	val[0] = (Scr.VxMax + Scr.MyDisplayWidth) / Scr.MyDisplayWidth;
@@ -679,7 +697,7 @@ void
 GNOME_SetCurrentArea(void)
 {
 	Atom atom_set;
-	CARD32 val[2];
+	long val[2];
 
 	atom_set = XInternAtom(dpy, XA_WIN_AREA, False);
 	val[0] = Scr.Vx / Scr.MyDisplayWidth;
@@ -697,7 +715,7 @@ void
 GNOME_SetDeskCount(void)
 {
 	Atom atom_set;
-	CARD32 val;
+	long val;
 	FvwmWindow *t;
 
 	atom_set = XInternAtom(dpy, XA_WIN_WORKSPACE_COUNT, False);
@@ -732,10 +750,10 @@ void
 GNOME_SetCurrentDesk(void)
 {
 	Atom atom_set;
-	CARD32 val;
+	long val;
 
 	atom_set = XInternAtom(dpy, XA_WIN_WORKSPACE, False);
-	val = (CARD32) Scr.CurrentDesk;
+	val = (long) Scr.CurrentDesk;
 	if (val >= gnome_max_desk)
 	{
 		GNOME_SetDeskCount();
@@ -814,28 +832,28 @@ void
 GNOME_SetWinArea(FvwmWindow *w)
 {
 	Atom atom_set;
-	CARD32 val[2];
+	long val[2];
 
 	atom_set = XInternAtom(dpy, XA_WIN_AREA, False);
 
 	if (!DO_SKIP_WINDOW_LIST(w))
 	{
-		if (IsRectangleOnThisPage(&(w->frame_g), w->Desk))
+		if (IsRectangleOnThisPage(&(w->g.frame), w->Desk))
 		{
 			val[0] = Scr.Vx / Scr.MyDisplayWidth;
 			val[1] = Scr.Vy / Scr.MyDisplayHeight;
 		}
 		else
 		{
-			val[0] = (w->frame_g.x + Scr.Vx) / Scr.MyDisplayWidth;
+			val[0] = (w->g.frame.x + Scr.Vx) / Scr.MyDisplayWidth;
 			if (val[0] < 0 &&
-			    w->frame_g.x + Scr.Vx + w->frame_g.width > 0)
+			    w->g.frame.x + Scr.Vx + w->g.frame.width > 0)
 			{
 				val[0] = 0;
 			}
-			val[1] = (w->frame_g.y + Scr.Vy) / Scr.MyDisplayHeight;
+			val[1] = (w->g.frame.y + Scr.Vy) / Scr.MyDisplayHeight;
 			if (val[1] < 0 &&
-			    w->frame_g.y + Scr.Vy + w->frame_g.height > 0)
+			    w->g.frame.y + Scr.Vy + w->g.frame.height > 0)
 			{
 				val[1] = 0;
 			}
@@ -853,7 +871,7 @@ GNOME_Init(void)
 {
 	int i;
 	Atom atom_set, list[11];
-	CARD32 val;
+	long val;
 
 	atom_set = XInternAtom(dpy, XA_WIN_PROTOCOLS, False);
 	/* these indicate what GNOME compliance properties have been
@@ -924,14 +942,14 @@ CMD_GnomeShowDesks(F_CMD_ARGS)
 {
 	int n;
 	Atom atom_set;
-	CARD32 val[1];
+	long val[1];
 
 	atom_set = XInternAtom(dpy, XA_WIN_WORKSPACE_COUNT, False);
 	DBUG("GNOME_ShowDesks","Routine Entered");
 	n = GetIntegerArguments(action, NULL, (int *)val, 1);
 	if(n != 1)
 	{
-		fvwm_msg(ERR,"GnomeShowDesks","requires one argument");
+		fvwm_msg(ERR, "GnomeShowDesks", "requires one argument");
 		return;
 	}
 	gnome_max_desk = val[0];
@@ -947,12 +965,15 @@ CMD_GnomeShowDesks(F_CMD_ARGS)
 int
 GNOME_ProcessClientMessage(const exec_context_t *exc)
 {
-	int x, y;
 	Atom a;
-	FvwmWindow *fwin = exc->w.fw;
+	FvwmWindow *fw = exc->w.fw;
 	XEvent *ev = exc->x.etrigger;
+	long p0 = 0;
+	long p1 = 0;
+	const char* invalid_req = "---";
+	const char* invalid_prop = "---";
 
-	if (fwin != NULL  && DO_IGNORE_GNOME_HINTS(fwin))
+	if (fw != NULL  && DO_IGNORE_GNOME_HINTS(fw))
 	{
 		return 0;
 	}
@@ -960,33 +981,70 @@ GNOME_ProcessClientMessage(const exec_context_t *exc)
 	if (ev->xclient.message_type == a)
 	{
 		/* convert to integer grid */
-		x = ev->xclient.data.l[0] * Scr.MyDisplayWidth;
-		y = ev->xclient.data.l[1] * Scr.MyDisplayHeight;
+		p0 = ev->xclient.data.l[0] * Scr.MyDisplayWidth;
+		p1 = ev->xclient.data.l[1] * Scr.MyDisplayHeight;
+		if (p0 < 0 || p0 > 0x7fffffff || p1 < 0 || p1 > 0x7fffffff)
+		{
+			invalid_prop = "page";
+			invalid_req = "_WIN_AREA";
+			goto err_msg;
+		}
+		MoveViewport(p0, p1, 1);
 
-		MoveViewport(x, y, 1);
 		return 1;
 	}
 	a = XInternAtom(dpy, XA_WIN_WORKSPACE, False);
 	if (ev->xclient.message_type == a)
 	{
-		goto_desk(ev->xclient.data.l[0]);
+		p0 = ev->xclient.data.l[0];
+		if (p0 < 0 || p0 > 0x7fffffff)
+		{
+			invalid_prop = "desk";
+			invalid_req = "_WIN_WORKSPACE";
+			goto err_msg;
+		}
+		goto_desk(p0);
+
 		return 1;
 	}
 	a = XInternAtom(dpy, XA_WIN_LAYER, False);
-	if (ev->xclient.message_type == a && fwin)
+	if (ev->xclient.message_type == a && fw)
 	{
-		new_layer(fwin, ev->xclient.data.l[0]);
+		p0 = ev->xclient.data.l[0];
+		if (p0 < 0 || p0 > 0x7fffffff)
+		{
+			invalid_prop = "layer";
+			invalid_req = "_WIN_LAYER";
+			goto err_msg;
+		}
+		new_layer(fw, p0);
+
 		return 1;
 	}
 	a = XInternAtom(dpy, XA_WIN_STATE, False);
-	if (ev->xclient.message_type == a && fwin)
+	if (ev->xclient.message_type == a && fw)
 	{
 		GNOME_HandlePropRequest(
 			exc, ev->xclient.data.l[0], ev->xclient.data.l[1]);
+
 		return 1;
 	}
 
 	return 0;
+
+  err_msg:
+	fvwm_msg(
+		WARN, "GNOME_ProcessClientMessage",
+		"The application window (id %#lx)\n"
+		"  \"%s\" has requested an invalid %s (%ld, %ld)\n"
+		"  using a %s client message.\n"
+		"    fvwm is ignoring this request.\n",
+		fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)", invalid_prop,
+		p0, p1, invalid_req);
+	fvwm_msg_report_app_and_workers();
+
+	return 0;
+
 }
 
 void GNOME_HandlePropRequest(
@@ -1012,7 +1070,7 @@ void GNOME_HandlePropRequest(
 	}
 	/*--------------------------------------------------------------------
 	  First change the props on the window so the gnome app knows we did
-	  somthing.
+	  something.
 	  --------------------------------------------------------------------*/
 #ifdef FVWM_DEBUG_MSGS
 	fvwm_msg(
@@ -1069,7 +1127,7 @@ void GNOME_HandlePropRequest(
 	}
 	/*--------------------------------------------------------------------
 	  WindowShade
-	  --------------------------------------------------------------------*/
+	  -------------------------------------------------------------------*/
 	if (propm & WIN_STATE_SHADED)
 	{
 		if (prop & WIN_STATE_SHADED)
@@ -1121,4 +1179,3 @@ CMD_GnomeButton(F_CMD_ARGS)
 
 	return;
 }
-
