@@ -14,12 +14,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
 #include "config.h"
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
+#include "libs/charmap.h"
+#include "libs/wcontext.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "execcontext.h"
@@ -41,26 +43,27 @@
 #include "module_interface.h"
 #include "focus.h"
 #include "stack.h"
+#include "icons.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 static void init_style(
 	FvwmWindow *old_t, FvwmWindow *t, window_style *pstyle,
@@ -70,6 +73,10 @@ static void init_style(
 	memcpy(old_t, t, sizeof(FvwmWindow));
 	/* determine level of decoration */
 	setup_style_and_decor(t, pstyle, pbuttons);
+	/* restore some old values */
+	GET_USER_STATES(t) = GET_USER_STATES(old_t);
+
+	return;
 }
 
 static void apply_window_updates(
@@ -83,15 +90,15 @@ static void apply_window_updates(
 	const exec_context_t *exc;
 	exec_context_changes_t ecc;
 
-	frame_g.x = t->frame_g.x;
-	frame_g.y = t->frame_g.y;
-	frame_g.width = t->frame_g.width;
-	frame_g.height = t->frame_g.height;
+	frame_g.x = t->g.frame.x;
+	frame_g.y = t->g.frame.y;
+	frame_g.width = t->g.frame.width;
+	frame_g.height = t->g.frame.height;
 	if (flags->do_setup_focus_policy)
 	{
 		setup_focus_policy(t);
 		if (t == focus_w &&
-		    fpol_query_allow_user_focus(&FW_FOCUS_POLICY(t)))
+		    !fpol_query_allow_user_focus(&FW_FOCUS_POLICY(t)))
 		{
 			focus_w = NULL;
 			if (Scr.Hilite == t)
@@ -131,29 +138,26 @@ static void apply_window_updates(
 	exc = exc_create_context(
 		&ecc, ECC_TYPE | ECC_FW | ECC_W | ECC_WCONTEXT);
 	if (flags->do_update_stick_icon && IS_ICONIFIED(t) &&
-	    !(IS_STICKY_ON_PAGE(t) || IS_STICKY_ON_DESK(t)))
+	    !(IS_STICKY_ACROSS_PAGES(t) || IS_STICKY_ACROSS_DESKS(t)))
 	{
-		if (IS_ICON_STICKY_ON_PAGE(pstyle) ||
-		    IS_ICON_STICKY_ON_DESK(pstyle))
+		if (IS_ICON_STICKY_ACROSS_PAGES(pstyle) ||
+		    IS_ICON_STICKY_ACROSS_DESKS(pstyle))
 		{
 			/* stick and unstick the window to force the icon on
 			 * the current page */
 			handle_stick(
 				NULL, exc, "",
-				S_IS_STICKY_ON_PAGE(SCF(*pstyle)),
-				S_IS_STICKY_ON_DESK(SCF(*pstyle)), 1, 1);
+				S_IS_STICKY_ACROSS_PAGES(SCF(*pstyle)),
+				S_IS_STICKY_ACROSS_DESKS(SCF(*pstyle)), 1, 1);
 			handle_stick(NULL, exc, "", 0, 0, 1, 0);
 		}
-		else
-		{
-			flags->do_update_icon_title = True;
-		}
+		flags->do_update_icon_title = True;
 	}
 	else if (flags->do_update_stick)
 	{
 		handle_stick(
-			NULL, exc, "", S_IS_STICKY_ON_PAGE(SCF(*pstyle)),
-			S_IS_STICKY_ON_DESK(SCF(*pstyle)), 0, 0);
+			NULL, exc, "", S_IS_STICKY_ACROSS_PAGES(SCF(*pstyle)),
+			S_IS_STICKY_ACROSS_DESKS(SCF(*pstyle)), 0, 0);
 	}
 	exc_destroy_context(exc);
 	if (FMiniIconsSupported && flags->do_update_mini_icon)
@@ -192,8 +196,7 @@ static void apply_window_updates(
 		EWMH_SetVisibleName(t, True);
 	}
 
-	if (flags->do_update_window_font ||
-	    flags->do_update_window_font_height)
+	if (flags->do_update_window_font || flags->do_update_window_font_height)
 	{
 		if (!is_style_initialised)
 		{
@@ -203,6 +206,10 @@ static void apply_window_updates(
 		setup_window_font(t, pstyle, flags->do_update_window_font);
 		flags->do_redecorate = True;
 	}
+	if (flags->do_update_title_text_dir)
+	{
+		flags->do_redecorate = True;
+	}
 	if (flags->do_redecorate || flags->do_update_title_dir)
 	{
 		size_borders b_old;
@@ -227,14 +234,14 @@ static void apply_window_updates(
 			/* naked_g: geometry without decor */
 			gravity_get_naked_geometry(
 				old_t.hints.win_gravity, &old_t, &naked_g,
-				&t->normal_g);
+				&t->g.normal);
 			/* gravity without decor */
 			gravity_translate_to_northwest_geometry_no_bw(
 				old_t.hints.win_gravity, &old_t, &naked_g,
 				&naked_g);
-			/* set normal_g with the decor */
+			/* set g.normal with the decor */
 			gravity_add_decoration(
-				old_t.hints.win_gravity, t, &t->normal_g,
+				old_t.hints.win_gravity, t, &t->g.normal,
 				&naked_g);
 		}
 		if (flags->do_update_title_dir)
@@ -249,17 +256,17 @@ static void apply_window_updates(
 			dw = b_new.total_size.width - b_old.total_size.width;
 			dh = b_new.total_size.height - b_old.total_size.height;
 			gravity_resize(
-				t->hints.win_gravity, &t->normal_g, dw, dh);
+				t->hints.win_gravity, &t->g.normal, dw, dh);
 			gravity_constrain_size(
-				t->hints.win_gravity, t, &t->normal_g, 0);
+				t->hints.win_gravity, t, &t->g.normal, 0);
 		}
 
 		if (IS_MAXIMIZED(t))
 		{
 			if (flags->do_redecorate)
 			{
-				int off_x = old_t.normal_g.x - old_t.max_g.x;
-				int off_y = old_t.normal_g.y - old_t.max_g.y;
+				int off_x = old_t.g.normal.x - old_t.g.max.x;
+				int off_y = old_t.g.normal.y - old_t.g.max.y;
 				int new_off_x;
 				int new_off_y;
 
@@ -267,35 +274,35 @@ static void apply_window_updates(
 				 * have NorthWestGravity */
 				gravity_get_naked_geometry(
 					NorthWestGravity, &old_t, &naked_g,
-					&t->max_g);
+					&t->g.max);
 				gravity_translate_to_northwest_geometry_no_bw(
 					NorthWestGravity, &old_t, &naked_g,
 					&naked_g);
 				gravity_add_decoration(
-					NorthWestGravity, t, &t->max_g,
+					NorthWestGravity, t, &t->g.max,
 					&naked_g);
-				/* prevent random paging when unmaximizing after
-				 * e.g.the border width has changed */
-				new_off_x = t->normal_g.x - t->max_g.x;
-				new_off_y = t->normal_g.y - t->max_g.y;
-				t->max_offset.x += new_off_x - off_x;
-				t->max_offset.y += new_off_y - off_y;
+				/* prevent random paging when unmaximizing
+				 * after e.g. the border width has changed */
+				new_off_x = t->g.normal.x - t->g.max.x;
+				new_off_y = t->g.normal.y - t->g.max.y;
+				t->g.max_offset.x += new_off_x - off_x;
+				t->g.max_offset.y += new_off_y - off_y;
 			}
 			if (flags->do_update_title_dir)
 			{
-				frame_g = t->max_g;
+				frame_g = t->g.max;
 				gravity_resize(
-					t->hints.win_gravity, &t->max_g, dw,
+					t->hints.win_gravity, &t->g.max, dw,
 					dh);
 				gravity_constrain_size(
-					t->hints.win_gravity, t, &t->max_g,
+					t->hints.win_gravity, t, &t->g.max,
 					CS_UPDATE_MAX_DEFECT);
 			}
-			new_g = &t->max_g;
+			new_g = &t->g.max;
 		}
 		else
 		{
-			new_g = &t->normal_g;
+			new_g = &t->g.normal;
 		}
 		if (IS_SHADED(t))
 		{
@@ -313,59 +320,13 @@ static void apply_window_updates(
 		flags->do_setup_frame = True;
 		flags->do_redraw_decoration = True;
 	}
-	if (flags->do_update_title_text_dir)
+	if (flags->do_update_rotated_title)
 	{
-		if (!flags->do_update_title_dir)
+		if (t->title_text_rotation != ROTATION_0)
 		{
-			setup_title_geometry(t, pstyle);
+			flags->do_setup_frame = True;
+			flags->do_redraw_decoration = True;
 		}
-		flags->do_redraw_decoration = True;
-	}
-	if (0 && flags->do_update_title_dir)
-	{
-		size_borders b_old;
-		size_borders b_new;
-		rectangle unshaded_g;
-		int dw;
-		int dh;
-
-		get_unshaded_geometry(t, &unshaded_g);
-		get_window_borders(t, &b_old);
-		SET_TITLE_DIR(t, S_TITLE_DIR(SCF(*pstyle)));
-		setup_title_geometry(t, pstyle);
-		get_window_borders(t, &b_new);
-		dw = b_new.total_size.width - b_old.total_size.width;
-		dh = b_new.total_size.height - b_old.total_size.height;
-
-		frame_g = t->normal_g;
-		gravity_resize(t->hints.win_gravity, &frame_g, dw, dh);
-		gravity_constrain_size(t->hints.win_gravity, t, &frame_g, 0);
-		t->normal_g = frame_g;
-		if (IS_MAXIMIZED(t))
-		{
-			frame_g = t->max_g;
-			gravity_resize(
-				t->hints.win_gravity, &frame_g, dw, dh);
-			gravity_constrain_size(
-				t->hints.win_gravity, t, &frame_g,
-				CS_UPDATE_MAX_DEFECT);
-			t->max_g = frame_g;
-		}
-		if (IS_SHADED(t))
-		{
-			get_shaded_geometry(t, &frame_g, &unshaded_g);
-		}
-		else if (IS_MAXIMIZED(t))
-		{
-			get_relative_geometry(&frame_g, &t->max_g);
-		}
-		else
-		{
-			get_relative_geometry(&frame_g, &t->normal_g);
-		}
-
-		flags->do_setup_frame = True;
-		flags->do_redraw_decoration = True;
 	}
 	if (flags->do_resize_window)
 	{
@@ -373,16 +334,16 @@ static void apply_window_updates(
 
 		setup_frame_size_limits(t, pstyle);
 		old_g = frame_g;
-		frame_g = t->normal_g;
+		frame_g = t->g.normal;
 		gravity_constrain_size(t->hints.win_gravity, t, &frame_g, 0);
-		t->normal_g = frame_g;
+		t->g.normal = frame_g;
 		if (IS_MAXIMIZED(t))
 		{
-			frame_g = t->max_g;
+			frame_g = t->g.max;
 			gravity_constrain_size(
 				t->hints.win_gravity, t, &frame_g,
 				CS_UPDATE_MAX_DEFECT);
-			t->max_g = frame_g;
+			t->g.max = frame_g;
 		}
 		frame_g = old_g;
 		gravity_constrain_size(
@@ -432,23 +393,36 @@ static void apply_window_updates(
 			flags->do_broadcast_focus = True;
 		}
 	}
-	if (flags->do_redraw_decoration)
+	if (flags->do_update_icon_title_cs_hi)
 	{
-		FvwmWindow *tmp;
-
-		/* frame_redraw_decorations needs to know if the window is
-		 * hilighted */
-		tmp = get_focus_window();
-		set_focus_window(focus_w);
-		if (IS_ICONIFIED(t))
+		if (t == focus_w && IS_ICONIFIED(t))
 		{
-			DrawIconWindow(t, True, True, False, NULL);
+			flags->do_redraw_icon = True;
+		}
+		update_icon_title_cs_hi_style(t, pstyle);
+	}
+	if (flags->do_update_icon_title_cs)
+	{
+		if (t != focus_w && IS_ICONIFIED(t))
+		{
+			flags->do_redraw_icon = True;
+		}
+		update_icon_title_cs_style(t, pstyle);
+	}
+	if (flags->do_update_icon_background_cs)
+	{
+		int old_cs = t->icon_background_cs;
+
+		update_icon_background_cs_style(t, pstyle);
+		if ((old_cs < 0 && t->icon_background_cs >= 0) ||
+		    (old_cs >= 0 && t->icon_background_cs < 0))
+		{
+			flags->do_update_icon = True;
 		}
 		else
 		{
-			border_redraw_decorations(t);
+			flags->do_redraw_icon = True;
 		}
-		set_focus_window(tmp);
 	}
 	if (flags->do_update_icon_size_limits)
 	{
@@ -471,32 +445,55 @@ static void apply_window_updates(
 	}
 	if (flags->do_update_icon)
 	{
+		setup_icon_background_parameters(t, pstyle);
+		setup_icon_title_parameters(t, pstyle);
 		change_icon(t, pstyle);
 		flags->do_update_icon_placement = True;
 		flags->do_update_icon_title = False;
+		flags->do_redraw_icon = False;
 		flags->do_update_ewmh_icon = True;
 	}
+	if (flags->do_redraw_icon)
+	{
+		/* should not test if the window is iconified */
+		DrawIconWindow(t, True, True, False, True, NULL);;
+		flags->do_redraw_decoration = False;
+		flags->do_update_icon_title = False;
+	}
 	if (flags->do_update_icon_title)
 	{
 		RedoIconName(t);
-#if 0
-		if (IS_ICONIFIED(t))
-		{
-			DrawIconWindow(t, True, True, False, NULL);
-		}
-#endif
 	}
 	if (flags->do_update_icon_placement)
 	{
 		if (IS_ICONIFIED(t))
 		{
-			initial_window_options_type win_opts;
+			initial_window_options_t win_opts;
 
 			memset(&win_opts, 0, sizeof(win_opts));
 			SET_ICONIFIED(t, 0);
 			Iconify(t, &win_opts);
+			flags->do_redraw_decoration = False;
 		}
 	}
+	if (flags->do_redraw_decoration)
+	{
+		FvwmWindow *tmp;
+
+		/* frame_redraw_decorations needs to know if the window is
+		 * hilighted */
+		tmp = get_focus_window();
+		set_focus_window(focus_w);
+		if (IS_ICONIFIED(t))
+		{
+			DrawIconWindow(t, True, True, False, False, NULL);
+		}
+		else
+		{
+			border_redraw_decorations(t);
+		}
+		set_focus_window(tmp);
+	}
 	if (flags->do_update_frame_attributes)
 	{
 		setup_frame_attributes(t, pstyle);
@@ -530,14 +527,22 @@ static void apply_window_updates(
 			if (t->ewmh_hint_layer > 0 &&
 			    t->layer != t->ewmh_hint_layer)
 			{
+				t->ewmh_normal_layer = t->layer;
 				new_layer(t, t->ewmh_hint_layer);
 			}
 		}
 		else
 		{
-			if (t->ewmh_hint_layer > 0)
+			if (t->ewmh_hint_layer > 0 && t->ewmh_normal_layer)
 			{
-				new_layer(t, Scr.DefaultLayer);
+				if (t->ewmh_normal_layer)
+				{
+					new_layer(t, t->ewmh_normal_layer);
+				}
+				else
+				{
+					new_layer(t, Scr.DefaultLayer);
+				}
 			}
 		}
 	}
@@ -550,10 +555,10 @@ static void apply_window_updates(
 		if (Scr.Hilite != NULL && t == Scr.Hilite)
 		{
 			BroadcastPacket(
-				M_FOCUS_CHANGE, 5, FW_W(Scr.Hilite),
-				FW_W_FRAME(Scr.Hilite), 0,
-				Scr.Hilite->hicolors.fore,
-				Scr.Hilite->hicolors.back);
+				M_FOCUS_CHANGE, 5, (long)FW_W(Scr.Hilite),
+				(long)FW_W_FRAME(Scr.Hilite), (long)0,
+				(long)Scr.Hilite->hicolors.fore,
+				(long)Scr.Hilite->hicolors.back);
 		}
 	}
 	if (flags->do_refresh)
@@ -564,17 +569,35 @@ static void apply_window_updates(
 		}
 	}
 	t->shade_anim_steps = pstyle->shade_anim_steps;
+	if (flags->do_update_cr_motion_method)
+	{
+		switch (SCR_MOTION_METHOD(&pstyle->flags))
+		{
+		case WS_CR_MOTION_METHOD_AUTO:
+			if (WAS_CR_MOTION_METHOD_DETECTED(t))
+			{
+				/* method was already detected, keep it */
+				break;
+			}
+			/* fall through */
+		case WS_CR_MOTION_METHOD_USE_GRAV:
+		case WS_CR_MOTION_METHOD_STATIC_GRAV:
+			SET_CR_MOTION_METHOD(
+				t, SCR_MOTION_METHOD(&pstyle->flags));
+			SET_CR_MOTION_METHOD_DETECTED(t, 0);
+			break;
+		}
+	}
 
 	return;
 }
 
-/* ---------------------------- builtin commands ---------------------------- */
+/* ---------------------------- builtin commands --------------------------- */
 
 /* takes only care of destroying windows that have to go away. */
 void destroy_scheduled_windows(void)
 {
-	FvwmWindow *t;
-	FvwmWindow *next;
+	flist *t;
 	Bool do_need_ungrab = False;
 
 	if (Scr.flags.is_executing_complex_function ||
@@ -592,15 +615,11 @@ void destroy_scheduled_windows(void)
 	Scr.flags.is_window_scheduled_for_destroy = 0;
 	/* need to destroy one or more windows before looking at the window
 	 * list */
-	for (t = Scr.FvwmRoot.next, next = NULL; t != NULL; t = next)
+	for (t = Scr.FWScheduledForDestroy; t != NULL; t = t->next)
 	{
-		next = t->next;
-		if (IS_SCHEDULED_FOR_DESTROY(t))
-		{
-			SET_SCHEDULED_FOR_DESTROY(t, 0);
-			destroy_window(t);
-		}
+		destroy_window(t->object);
 	}
+	Scr.FWScheduledForDestroy = flist_free_list(Scr.FWScheduledForDestroy);
 	MyXUngrabServer(dpy);
 	if (do_need_ungrab)
 	{
