@@ -13,14 +13,13 @@
 #include <stdio.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include "time.h"
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <signal.h>
 #include <errno.h>
 
-#include <libs/fvwmlib.h>
-#include <libs/Module.h>
+#include "libs/fvwmlib.h"
+#include "libs/Module.h"
 #include "libs/fvwmsignal.h"
 
 #define S_NAME  "/.FvwmConsole-Socket"
@@ -34,9 +33,6 @@
 /* #define M_PASS M_ERROR */
 #define M_PASS M_ERROR
 
-/* number of default arguments when invoked from fvwm */
-#define FARGS 6
-
 #define XTERM "xterm"
 
 /* message to client */
